@@ -1,22 +1,33 @@
-// vamps - eVAporate DVD compliant Mpeg2 Program Stream
+// Vamps - eVAporate DVD compliant Mpeg2 Program Stream
 //
-// This program is free software; you can redistribute it and/or modify
+// This file is part of Vamps.
+//
+// Vamps is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
+// the Free Software Foundation; version 2 of the License.
 //
-// This program is distributed in the hope that it will be useful,
+// Vamps is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+// along with Vamps; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 //
 // Revision history (latest first):
 //
+// 2006/04/15: V0.99.2: Fixed some signed/unsigned issues which caused compiler
+//                      warnings on some platforms. No funtional changes.
+//
+// 2006/02/26: V0.99.1: Introduced detection of special packets inserted by
+//                      play_cell when it encounters a cell gap. This enables
+//                      proper evaporation of some strangely authored DVDs.
+//
+// 2006/02/14: V0.99: Usage of inttypes.h. No functional changes (however,
+//		      play_cell *is* modified).
+//
 // 2005/11/06: V0.98: Updated requant module to
 //		      latest version from Metakine web site
 //		      (http://www.metakine.com/files/M2VRequantiser.tgz).
@@ -55,7 +66,7 @@
 // for large file support (>2GB)
 #define _GNU_SOURCE
 #define _FILE_OFFSET_BITS	64
-#define VERSION			"V0.98"
+#define VERSION			"V0.99.2"
 
 #include <stdio.h>
 #include <errno.h>
@@ -65,6 +76,7 @@
 #include <string.h>
 #include <getopt.h>
 #include <pthread.h>
+#include <inttypes.h>
 #include <sys/stat.h>
 
 // DVD sector size
@@ -80,26 +92,20 @@
 #define VBUF_SIZE (1024*1024)
 
 
-// shortcuts for some frequently used types
-typedef unsigned char      UInt8;
-typedef unsigned long      UInt32;
-typedef unsigned long long UInt64;
-
-
 // a lot of global data - we really should use C++...
-UInt8       rbuf  [RBUF_SIZE];		// the PS read buffer
-UInt8       wbuf  [WBUF_SIZE];		// the PS write buffer
-UInt8      *vibuf;			// the video ES requant input buffer
-UInt8      *vobuf;			// the video ES requant output buffer
-UInt8      *rptr = rbuf;		// pointer to current char in read buf
-UInt8      *rhwp = rbuf;		// read buffer high water pointer
-UInt8      *wptr = wbuf;		// pointer to first unused char in wbuf
-UInt64      bytes_read;			// total PS bytes read
-UInt64      bytes_written;		// total PS bytes written
-UInt64      padding_bytes;		// total padding bytes written
-UInt64      vin_bytes;			// total unshrinked video ES bytes
-UInt64      vout_bytes;			// total shrinked video ES bytes
-UInt64      ps_size;			// total PS size in bytes
+uint8_t     rbuf  [RBUF_SIZE];		// the PS read buffer
+uint8_t     wbuf  [WBUF_SIZE];		// the PS write buffer
+uint8_t    *vibuf;			// the video ES requant input buffer
+uint8_t    *vobuf;			// the video ES requant output buffer
+uint8_t    *rptr = rbuf;		// pointer to current char in read buf
+uint8_t    *rhwp = rbuf;		// read buffer high water pointer
+uint8_t    *wptr = wbuf;		// pointer to first unused char in wbuf
+uint64_t    bytes_read;			// total PS bytes read
+uint64_t    bytes_written;		// total PS bytes written
+uint64_t    padding_bytes;		// total padding bytes written
+uint64_t    vin_bytes;			// total unshrinked video ES bytes
+uint64_t    vout_bytes;			// total shrinked video ES bytes
+uint64_t    ps_size;			// total PS size in bytes
 int         vbuf_size = VBUF_SIZE;	// the video ES requant buffers' size
 int         vilen;			// current GOP's unshrinked vidES bytes
 int         volen;			// current GOP's shrinked vidES bytes
@@ -108,16 +114,17 @@ int         video_packs;		// no. video packs in PS
 int         skipped_video_packs;	// skipped thereof
 int         aux_packs;			// no. audio and subpicture packs in PS
 int         skipped_aux_packs;		// skipped thereof
+int         cell_gap_packs;		// sectors skipped reported by play_cell
 int         sequence_headers;		// no. sequence headers (== #GOPs)
 int         nav_packs;			// no. nav packs
 int         eof;			// end of file flag
-UInt8      *rqt_rptr;			// ptr to cur char in requant read buf
-UInt8      *rqt_wptr;			// ptr to first unused char in rqt wbuf
+uint8_t    *rqt_rptr;			// ptr to cur char in requant read buf
+uint8_t    *rqt_wptr;			// ptr to first unused char in rqt wbuf
 int         rqt_rcnt;			// no. bytes in requant read buffer
 int         rqt_wcnt;			// no. bytes in requant write buffer
-UInt64      rqt_inbytes;		// total bytes fed into requantization
-UInt64      rqt_outbytes;		// total bytes melt from requantization
-UInt64      rqt_visize;			// calc. total unshrinked vidES bytes
+uint64_t    rqt_inbytes;		// total bytes fed into requantization
+uint64_t    rqt_outbytes;		// total bytes melt from requantization
+uint64_t    rqt_visize;			// calc. total unshrinked vidES bytes
 float       rqt_fact;			// video ES vaporization factor (>1)
 int         spu_track_map [32];		// subpicture track# translation map
 int         audio_track_map [8];	// audio track# translation map
@@ -150,6 +157,7 @@ struct
   inject_int    (skipped_video_packs),
   inject_int    (aux_packs),
   inject_int    (skipped_aux_packs),
+  inject_int    (cell_gap_packs),
   inject_int    (sequence_headers),
   inject_int    (nav_packs)
 };
@@ -162,7 +170,7 @@ pthread_mutex_t mutw  = PTHREAD_MUTEX_INITIALIZER;
 
 
 // prototypes
-void scan_track_map (int *, int);
+void scan_track_map (int *, unsigned);
 void vaporize (void);
 void read_injections (const char *);
 void write_injections (const char *);
@@ -198,7 +206,8 @@ main (int argc, char *const argv [])
   while ((c = getopt_long_only (argc, argv, "a:s:e:E:vi:pS:",
 				long_options, NULL)) >= 0)
   {
-    switch (c) {
+    switch (c)
+    {
       case 'a':
 	scan_track_map (audio_track_map, 8);
 	break;
@@ -315,12 +324,17 @@ main (int argc, char *const argv [])
 	     nav_packs);
     fprintf (stderr,
 "Info: Total bytes vaporized in audio and subpicture streams: %llu (%.2f%%)\n",
-	     (UInt64) skipped_aux_packs * SECT_SIZE,
+	     (uint64_t) skipped_aux_packs * SECT_SIZE,
 	     (double) skipped_aux_packs /
 	     (double) (bytes_read / SECT_SIZE) * 100.0);
+    fprintf (stderr,
+	     "Info: Total bytes vaporized in cell gaps: %llu (%.2f%%)\n",
+	     (uint64_t) cell_gap_packs * SECT_SIZE,
+	     (double) cell_gap_packs /
+	     (double) (bytes_read / SECT_SIZE) * 100.0);
     fprintf (stderr,
 	     "Info: Total bytes vaporized in video stream: %llu (%.2f%%)\n",
-	     (UInt64) skipped_video_packs * SECT_SIZE,
+	     (uint64_t) skipped_video_packs * SECT_SIZE,
 	     (double) skipped_video_packs /
 	     (double) (bytes_read / SECT_SIZE) * 100.0);
     fprintf (stderr,
@@ -343,9 +357,9 @@ main (int argc, char *const argv [])
 
 // analyze track numbers given in cmd line ("x,y,z")
 void
-scan_track_map (int *map, int ntracks)
+scan_track_map (int *map, unsigned ntracks)
 {
-  int      i = 1;
+  unsigned i = 1;
   unsigned track;
   char    *op = optarg;
 
@@ -438,8 +452,8 @@ flush (void)
 
 
 // returns no. bytes read up to where `ptr' points
-static inline UInt64
-rtell (UInt8 *ptr)
+static inline uint64_t
+rtell (uint8_t *ptr)
 {
   return bytes_read - (rhwp - ptr);
 }
@@ -447,8 +461,8 @@ rtell (UInt8 *ptr)
 
 // returns no. bytes written up to where `ptr' points
 // (including those in buffer which are not actually written yet)
-static inline UInt64
-wtell (UInt8 *ptr)
+static inline uint64_t
+wtell (uint8_t *ptr)
 {
   return bytes_written + (ptr - wbuf);
 }
@@ -456,15 +470,15 @@ wtell (UInt8 *ptr)
 
 // some pack header consistency checking
 static inline void
-check_pack (UInt8 *ptr)
+check_pack (uint8_t *ptr)
 {
-  UInt32 pack_start_code;
-  int    pack_stuffing_length;
+  uint32_t pack_start_code;
+  int      pack_stuffing_length;
 
-  pack_start_code  = (UInt32) (ptr [0]) << 24;
-  pack_start_code |= (UInt32) (ptr [1]) << 16;
-  pack_start_code |= (UInt32) (ptr [2]) <<  8;
-  pack_start_code |= (UInt32) (ptr [3]);
+  pack_start_code  = (uint32_t) (ptr [0]) << 24;
+  pack_start_code |= (uint32_t) (ptr [1]) << 16;
+  pack_start_code |= (uint32_t) (ptr [2]) <<  8;
+  pack_start_code |= (uint32_t) (ptr [3]);
 
   if (pack_start_code != 0x000001ba)
     fatal ("Bad pack start code at %llu: %08lx", rtell (ptr), pack_start_code);
@@ -484,15 +498,15 @@ check_pack (UInt8 *ptr)
 
 // video packet consistency checking
 static inline int
-check_video_packet (UInt8 *ptr)
+check_video_packet (uint8_t *ptr)
 {
-  int    vid_packet_length, pad_packet_length, rc = 0;
-  UInt32 vid_packet_start_code, pad_packet_start_code, sequence_header_code;
+  int      vid_packet_length, pad_packet_length, rc = 0;
+  uint32_t vid_packet_start_code, pad_packet_start_code, sequence_header_code;
 
-  vid_packet_start_code  = (UInt32) (ptr [0]) << 24;
-  vid_packet_start_code |= (UInt32) (ptr [1]) << 16;
-  vid_packet_start_code |= (UInt32) (ptr [2]) <<  8;
-  vid_packet_start_code |= (UInt32) (ptr [3]);
+  vid_packet_start_code  = (uint32_t) (ptr [0]) << 24;
+  vid_packet_start_code |= (uint32_t) (ptr [1]) << 16;
+  vid_packet_start_code |= (uint32_t) (ptr [2]) <<  8;
+  vid_packet_start_code |= (uint32_t) (ptr [3]);
 
   if (vid_packet_start_code != 0x000001e0)
     fatal ("Bad video packet start code at %llu: %08lx",
@@ -511,10 +525,10 @@ check_video_packet (UInt8 *ptr)
       fatal ("First video packet in sequence starting at %llu "
 	     "misses PTS or DTS, flags=%02x", rtell (ptr), ptr [7]);
 
-    sequence_header_code  = (UInt32) (ptr [6 + 3 + ptr [8] + 0]) << 24;
-    sequence_header_code |= (UInt32) (ptr [6 + 3 + ptr [8] + 1]) << 16;
-    sequence_header_code |= (UInt32) (ptr [6 + 3 + ptr [8] + 2]) <<  8;
-    sequence_header_code |= (UInt32) (ptr [6 + 3 + ptr [8] + 3]);
+    sequence_header_code  = (uint32_t) (ptr [6 + 3 + ptr [8] + 0]) << 24;
+    sequence_header_code |= (uint32_t) (ptr [6 + 3 + ptr [8] + 1]) << 16;
+    sequence_header_code |= (uint32_t) (ptr [6 + 3 + ptr [8] + 2]) <<  8;
+    sequence_header_code |= (uint32_t) (ptr [6 + 3 + ptr [8] + 3]);
 
     if (sequence_header_code == 0x000001b3)
     {
@@ -536,10 +550,10 @@ check_video_packet (UInt8 *ptr)
     // check for padding packet
     ptr += vid_packet_length;
 
-    pad_packet_start_code  = (UInt32) (ptr [0]) << 24;
-    pad_packet_start_code |= (UInt32) (ptr [1]) << 16;
-    pad_packet_start_code |= (UInt32) (ptr [2]) <<  8;
-    pad_packet_start_code |= (UInt32) (ptr [3]);
+    pad_packet_start_code  = (uint32_t) (ptr [0]) << 24;
+    pad_packet_start_code |= (uint32_t) (ptr [1]) << 16;
+    pad_packet_start_code |= (uint32_t) (ptr [2]) <<  8;
+    pad_packet_start_code |= (uint32_t) (ptr [3]);
 
     if (pad_packet_start_code != 0x000001be)
       fatal ("Bad padding packet start code at %llu: %08lx",
@@ -565,7 +579,7 @@ check_video_packet (UInt8 *ptr)
 // from a very short time) so a dual CPU box does not give an advantage
 // returns size of evaporated GOP
 static inline int
-requant (UInt8 *dst, UInt8 *src, int n, float fact)
+requant (uint8_t *dst, uint8_t *src, int n, float fact)
 {
   int rv;
 
@@ -581,8 +595,8 @@ requant (UInt8 *dst, UInt8 *src, int n, float fact)
   rqt_fact     = fact;
   rqt_inbytes  = vin_bytes;
   rqt_outbytes = vout_bytes;
-  rqt_visize   = (UInt64) ((float) ps_size * (float) vin_bytes /
- 			   ((float) total_packs * (float) SECT_SIZE));
+  rqt_visize   = (uint64_t) ((float) ps_size * (float) vin_bytes /
+			     ((float) total_packs * (float) SECT_SIZE));
 
   pthread_cond_signal (&condr);
   pthread_mutex_unlock (&mutr);
@@ -612,7 +626,7 @@ requant (UInt8 *dst, UInt8 *src, int n, float fact)
 // returns new track type (e.g. 0x80 for first AC3 audio
 // track in cmd line) or zero if track is not to be copied
 static inline int
-new_private_1_type (UInt8 *ptr)
+new_private_1_type (uint8_t *ptr)
 {
   int type, track, abase;
 
@@ -658,7 +672,7 @@ new_private_1_type (UInt8 *ptr)
 // patches track type to reflect new track
 // mapping unless user opted to preserve them
 static inline void
-copy_private_1 (UInt8 *ptr)
+copy_private_1 (uint8_t *ptr)
 {
   int type;
 
@@ -696,7 +710,7 @@ new_mpeg_audio_id (int id)
 // selectivly copy MPEG audio packs
 // patches ID to reflect new track mapping unless user opted to preserve them
 static inline void
-copy_mpeg_audio (UInt8 *ptr)
+copy_mpeg_audio (uint8_t *ptr)
 {
   int id;
 
@@ -725,8 +739,8 @@ copy_mpeg_audio (UInt8 *ptr)
 static void
 vap_leader (void)
 {
-  UInt8 *ptr;
-  int    id, data_length;
+  uint8_t *ptr;
+  int      id, data_length;
 
   while (!lock (SECT_SIZE))
   {
@@ -780,6 +794,15 @@ vap_leader (void)
 
 	break;
 
+      case 0xbf:
+	// private 2
+	if (strcmp ((char *) (ptr + 6), "Vamps-data") != 0 || ptr [17] != 0x01)
+	  fatal ("Misplaced private stream 2 pack at %llu", rtell (ptr));
+
+	skip (SECT_SIZE);
+
+	break;
+
       default:
 	fatal ("Encountered stream ID %02x at %llu, "
 	       "probably bad MPEG2 program stream", id, rtell (ptr));
@@ -801,8 +824,8 @@ vap_leader (void)
 static void
 vap_trailer (int length)
 {
-  UInt8 *ptr;
-  int    i, id, data_length;
+  uint8_t *ptr;
+  int      i, id, data_length;
 
   for (i = 0; i < length; i += SECT_SIZE)
   {
@@ -829,6 +852,14 @@ vap_trailer (int length)
 	fatal ("Bad padding packet length at %llu: %d",
 	       rtell (ptr), data_length);
     }
+    else if (id == 0xbf)
+    {
+      // private 2
+      if (strcmp ((char *) (ptr + 6), "Vamps-data") != 0 || ptr [17] != 0x01)
+	fatal ("Misplaced private stream 2 pack at %llu", rtell (ptr));
+
+      skip (SECT_SIZE);
+    }
     else
     {
       copy (SECT_SIZE);
@@ -853,8 +884,8 @@ vap_trailer (int length)
 static inline int
 vap_phase1 (void)
 {
-  UInt8 *ptr, *viptr = vibuf;
-  int    seq_length, id, data_length, opt_length, seqhdr;
+  uint8_t *ptr, *viptr = vibuf;
+  int      seq_length, id, data_length, opt_length, seqhdr, cell_gap;
 
   for (seq_length = 0;
        !lock (seq_length + SECT_SIZE); seq_length += SECT_SIZE)
@@ -960,6 +991,19 @@ vap_phase1 (void)
 
 	break;
 
+      case 0xbf:
+	// private 2
+	if (strcmp ((char *) (ptr + 6), "Vamps-data") != 0 || ptr [17] != 0x01)
+	  fatal ("Misplaced private stream 2 pack at %llu", rtell (ptr));
+
+	cell_gap        = ptr [18] << 8;
+	cell_gap       |= ptr [19];
+	cell_gap_packs += cell_gap--;
+	total_packs    += cell_gap;
+	bytes_read     += cell_gap * SECT_SIZE;
+
+	break;
+
       default:
 	fatal ("Encountered stream ID %02x at %llu, "
 	       "probably bad MPEG2 program stream", id, rtell (ptr));
@@ -978,7 +1022,7 @@ vap_phase1 (void)
 // `avail' specifies number of bytes remaining in vobuf
 // returns number of ES bytes in generated PES packet
 static inline int
-gen_video_packet (UInt8 *ptr, UInt8 *voptr, int avail)
+gen_video_packet (uint8_t *ptr, uint8_t *voptr, int avail)
 {
   int i, header_data_length, data_length, padding_length;
 
@@ -1044,8 +1088,8 @@ gen_video_packet (UInt8 *ptr, UInt8 *voptr, int avail)
 static inline void
 vap_phase2 (int seq_length)
 {
-  int    i, id, avail, data_length;
-  UInt8 *ptr, *voptr = vobuf, *vohwp = vobuf + volen;
+  int      i, id, avail, data_length;
+  uint8_t *ptr, *voptr = vobuf, *vohwp = vobuf + volen;
 
   for (i = 0; i < seq_length; i += SECT_SIZE)
   {
@@ -1106,6 +1150,15 @@ vap_phase2 (int seq_length)
 
 	break;
 
+      case 0xbf:
+	// private 2
+	if (strcmp ((char *) (ptr + 6), "Vamps-data") != 0 || ptr [17] != 0x01)
+	  fatal ("Misplaced private stream 2 pack at %llu", rtell (ptr));
+
+	skip (SECT_SIZE);
+
+	break;
+
       default:
 	fatal ("Encountered stream ID %02x at %llu, "
 	       "probably bad MPEG2 program stream", id, rtell (ptr));
@@ -1161,7 +1214,7 @@ vaporize (void)
       // forecast video ES vaporization factor
       // the basic formulars look like:
       // vap_fact  = total_packs/(restpacks+vop)
-      // restpacks = total_packs-(video_packs+skipped_aux_packs)
+      // restpacks = total_packs-(video_packs+skipped_aux_packs+cell_gap_packs)
       // fact      = (video_packs*net-(gops*net/2+10))/(vop*net-(gops*net/2+10))
       // net       = SECT_SIZE-(14+9)
       // 14: pack header size
@@ -1170,7 +1223,7 @@ vaporize (void)
       // You are welcome to double check everything here!
       float vop, net;
       net  = (float) (SECT_SIZE - (14+9));
-      vop  = video_packs + skipped_aux_packs -
+      vop  = video_packs + skipped_aux_packs + cell_gap_packs -
 	     (float) total_packs * (1.0f-1.0f/vap_fact);
       fact = ((float) video_packs * net -
 	      ((float) sequence_headers * net/2.0f + 10.0f)) /
@@ -1216,8 +1269,8 @@ vaporize (void)
 void
 read_injections (const char *filename)
 {
-  int   i;
-  FILE *fp;
+  unsigned i;
+  FILE    *fp;
 
   // lack of injections file is Ok
   // we will create one at end of this run
@@ -1238,8 +1291,8 @@ read_injections (const char *filename)
 void
 write_injections (const char *filename)
 {
-  int   i;
-  FILE *fp;
+  unsigned i;
+  FILE    *fp;
 
   if ((fp = fopen (filename, "w")) == NULL)
     fatal ("Failed creating injections file: %s: %s",
@@ -1250,19 +1303,19 @@ write_injections (const char *filename)
     switch (injections [i].s)
     {
 //      case 1:
-//	fprintf (fp, injections [i].f, *((UInt8 *) injections [i].p));
+//	fprintf (fp, injections [i].f, *((uint8_t *) injections [i].p));
 //	break;
 //
 //      case 2:
-//	fprintf (fp, injections [i].f, *((UInt16 *) injections [i].p));
+//	fprintf (fp, injections [i].f, *((uint16_t *) injections [i].p));
 //	break;
 //
       case 4:
-	fprintf (fp, injections [i].f, *((UInt32 *) injections [i].p));
+	fprintf (fp, injections [i].f, *((uint32_t *) injections [i].p));
 	break;
 
       case 8:
-	fprintf (fp, injections [i].f, *((UInt64 *) injections [i].p));
+	fprintf (fp, injections [i].f, *((uint64_t *) injections [i].p));
 	break;
 
       default:
@@ -1282,7 +1335,7 @@ void
 usage (void)
 {
   fputs (
-"vamps " VERSION "\n"
+"Vamps " VERSION "\n"
 "\n"
 "Usage: vamps [--evaporate|-e factor] [--ps-evaporate|-E factor]\n"
 "             [--audio|-a a-stream,a-stream,...]\n"
@@ -1294,7 +1347,7 @@ usage (void)
 "selectively copying audio and subpicture streams and by re-quantizing\n"
 "the embedded elementary video stream. The shrink factor may be either\n"
 "specified for the video ES only (-e) or for the full PS (-E).\n",
-  stderr);
+         stderr);
   exit (1);
 }
 
